@@ -7,7 +7,7 @@ features, stories, and validation results.
 
 from specfact_cli.models.deviation import Deviation, DeviationReport, DeviationSeverity, DeviationType, ValidationReport
 from specfact_cli.models.enforcement import EnforcementAction, EnforcementConfig, EnforcementPreset
-from specfact_cli.models.plan import Business, Feature, Idea, Metadata, PlanBundle, Product, Release, Story
+from specfact_cli.models.plan import Business, Feature, Idea, Metadata, PlanBundle, PlanSummary, Product, Release, Story
 from specfact_cli.models.protocol import Protocol, Transition
 
 
@@ -24,6 +24,7 @@ __all__ = [
     "Idea",
     "Metadata",
     "PlanBundle",
+    "PlanSummary",
     "Product",
     "Protocol",
     "Release",
